from setuptools import setup
setup(name='laplace',
      version='0.1',
      description='clinical trial information retriver',
      url='https://github.com/gastrodia/laplace',
      author='gastrodia',
      author_email='ipv6.elata@gmail.com',
      license='MIT',
      packages=['laplace','z3'],
      zip_safe=False)